import os
from dotenv import load_dotenv
load_dotenv()
os.environ['OPENAI_API_KEY'] = os.getenv('OPENAI_API_KEY')

import time
import shutil
import os
import PyPDF2

import pypdf
from docx import Document
from langchain.embeddings import SentenceTransformerEmbeddings
from langchain.embeddings import OpenAIEmbeddings
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain.chains import VectorDBQA
from langchain.chains import ConversationalRetrievalChain
from langchain.document_loaders import DirectoryLoader
from langchain.document_loaders import TextLoader
from langchain.chat_models import ChatOpenAI
from langchain.vectorstores import Chroma

query_model = "gpt-3.5-turbo"
embeddings = OpenAIEmbeddings() #SentenceTransformerEmbeddings(model_name="all-MiniLM-L6-v2")

database_directory = 'db/'
documents_directory = 'docs/'
all_files = 'docs_all'

def main():
    start_time = time.perf_counter()
    print("this runs")
    
    # convert_files_in_directory(all_files)

    # db = create_db_from_docs()
    # loader = DirectoryLoader(documents_directory)
    # documents = loader.load()
    # db.persist()
    # print(f"{len(documents)} documents were read in")

    query = "when was the latest revision made to the yageo sheet?"

    response = query_db(query)
    print(response)

    end_time = time.perf_counter()
    print(f"\nruntime = {(end_time - start_time):.3f} seconds")

def query_db(query):
    database = load_db()
    llm = ChatOpenAI(model_name=query_model)
    # VectorDBQA is Deprecated, but it works
    qa = VectorDBQA.from_chain_type(llm=llm, chain_type="stuff", vectorstore=database)
    #qa = ConversationalRetrievalChain.from_llm(ChatOpenAI(), database.as_retriever())
    return qa.run(query)

def create_db_from_docs():
    loader = DirectoryLoader(documents_directory)
    documents = loader.load()
    chunk_size=1000
    chunk_overlap=50

    text_splitter = RecursiveCharacterTextSplitter(chunk_size=chunk_size, chunk_overlap=chunk_overlap)
    docs = text_splitter.split_documents(documents)

    db = Chroma.from_documents(docs, embeddings, persist_directory=database_directory)
    return db

def add_to_db(file_path):
    file_path = convert_to_txt(file_path)

    loader = TextLoader(file_path)
    documents = loader.load()

    text_splitter = RecursiveCharacterTextSplitter(chunk_size=1000, chunk_overlap=50)
    texts = text_splitter.split_documents(documents)

    db = Chroma.from_documents(documents=texts, embedding=embeddings, persist_directory=database_directory)
    db.persist()
    
def load_db():
    return Chroma(persist_directory=database_directory, embedding_function=embeddings)

def delete_db():
    database = load_db()
    database.delete_collection()
    database.persist()
    
def convert_files_in_directory(dir_path):
    for filename in os.listdir(dir_path):
        filepath = os.path.join(dir_path, filename)
        if filename.endswith('.pdf'):
            pdf_to_txt(filepath)
        elif filename.endswith('.docx'):
            docx_to_txt(filepath)
        elif filename.endswith('.txt'):
            # place a copy of the txt file into the docs directory
            filename = os.path.basename(filename)
            filename_without_ext = os.path.splitext(filename)[0]
            destination = os.path.join(documents_directory, filename_without_ext + '.txt')
            
            # destination = f"{all_files}{filename}"
            shutil.copy(filepath, destination)

def convert_to_txt(file):
    if(is_txt(file)):
        return file
    
    elif(is_docx(file)):
        return docx_to_txt(file)

    elif(is_pdf(file)):
        return pdf_to_txt(file)

    else:
        print("The docs directory contains an unaccepted file type." +
        "accepted file types are: .txt, .docx, and .pdf")

def pdf_to_txt(filepath, dir_to_save_in=documents_directory):
    pdf_file = pypdf.PdfReader(open(filepath, 'rb'))
    num_pages = len(pdf_file.pages)

    filename = os.path.basename(filepath)
    filename_without_ext = os.path.splitext(filename)[0]
    print(f"{filename_without_ext}")

    txt_path = os.path.join(dir_to_save_in, filename_without_ext + '.txt')
    with open(txt_path, 'w', encoding='utf-8') as txt_file:
        for page_number in range(num_pages):
            page = pdf_file.pages[page_number]
            text = page.extract_text()
            
            #If any unknown characters are encountered, ignore them
            txt_file.write(text.encode('utf-8', errors='ignore').decode('utf-8'))

# def pdf_to_txt(filepath, dir_to_save_in=documents_directory):
#     # Open the PDF file in read-binary mode
#     txt_path = filepath[:-4] + '.txt'
#     with open(filepath, 'rb') as pdf_file:
#         # Create a PDF reader object
#         pdf_reader = PyPDF2.PdfReader(pdf_file)
        
#         # Extract text from each page
#         text = ""
#         for page in pdf_reader.pages:
#             text += page.extract_text()
        
#         # Create a file name for the text file
#         file_name = os.path.splitext(os.path.basename(filepath))[0]
#         txt_file_name = f"{file_name}.txt"
        
#         # Create the output file path
#         output_filepath = os.path.join(dir_to_save_in, txt_file_name)
        
#         # Write the extracted text to the output text file
#         with open(output_filepath, 'w', encoding='utf-8') as txt_file:
#             txt_file.write(text)
    
#     return txt_path

def docx_to_txt(filepath, dir_to_save_in=documents_directory):
    doc = Document(filepath)
    paragraphs = [p.text for p in doc.paragraphs]

    txt_path = filepath[:-5] + '.txt'
    with open(txt_path, 'w') as txt_file:
        txt_file.write('\n'.join(paragraphs))
    return txt_path

def is_txt(file_name):
    return file_name.endswith(".txt")

def is_pdf(file_name):
    return file_name.endswith(".pdf")

def is_docx(file_name):
    return file_name.endswith(".docx")

if __name__ == "__main__":
    main()