import os
from dotenv import load_dotenv
load_dotenv()
os.environ['OPENAI_API_KEY'] = os.getenv('OPENAI_API_KEY')
import openai
from langchain.embeddings.openai import OpenAIEmbeddings
from langchain.embeddings import SentenceTransformerEmbeddings
from langchain.document_loaders import DirectoryLoader
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain.vectorstores import Chroma

from langchain.chat_models import ChatOpenAI

from langchain.chains.question_answering import load_qa_chain

embeddings = SentenceTransformerEmbeddings(model_name="all-MiniLM-L6-v2")


def load_docs(directory):
    loader = DirectoryLoader(directory)
    documents = loader.load()
    return documents


def split_docs(documents,chunk_size=1000,chunk_overlap=20):
    text_splitter = RecursiveCharacterTextSplitter(chunk_size=chunk_size, chunk_overlap=chunk_overlap)
    docs = text_splitter.split_documents(documents)
    return docs

def main():
    '''directory = 'docs'
    documents = load_docs(directory)
    len(documents)

    docs = split_docs(documents)
    print(len(docs))

    embeddings = OpenAIEmbeddings()
    db = Chroma.from_documents(docs, embeddings)

    query = "What do different rulers do in North America?"
    matching_docs = db.similarity_search(query)

    print("printing matching documents:")

    print(matching_docs[0].page_content)
    print(matching_docs[1].page_content)
    #print(matching_docs[0].page_content)


    print("\nprinting similarity scores :")
    matching_docs = db.similarity_search_with_score(query,k=2)
    print(matching_docs[0])
    print()   
    print(matching_docs[1])
    print("\n\n")
 
    print("Now, let's repeat the above process with a Chroma database: ")

    persist_directory = "chroma_db"

    vectordb = Chroma.from_documents(
        documents=docs, embedding=embeddings, persist_directory=persist_directory
    )

    vectordb.persist('db')'''

    persist_directory = ".chroma"
    
     
    new_db = Chroma(persist_directory=persist_directory, embedding_function=embeddings)

    matching_docs = new_db.similarity_search_with_score(query)
    print(matching_docs[0])
    print()   
    print(matching_docs[1])
    print("\n\n")

#######################
#TESTED UP TO HERE
#####################
    print("\n\n")
    print("\n\n")
    print("starting untested code:")

    model_name = "gpt-3.5-turbo"
    llm = ChatOpenAI(model_name=model_name)

    # chain = load_qa_chain(llm, chain_type="stuff")
    chain = load_qa_chain(llm, chain_type="stuff",verbose=True)
    query = "What do American and Canadian rulers do?"
    matching_docs = db.similarity_search(query)
    answer =  chain.run(input_documents=matching_docs, question=query)
    print(answer)

    from langchain.chains import RetrievalQA
    retrieval_chain = RetrievalQA.from_chain_type(llm, chain_type="stuff", retriever=db.as_retriever())
    response = retrieval_chain.run(query)
    print(response)

if __name__ == "__main__":
    main()


     
    
    
