# https://github.com/logan-markewich/llama_index_starter_pack
# this code creates and edits a dummy database table
# this is used to supplement llama_db_interface.py to 
# experiment with how to create indeces in llama index from an SQL database

from multiprocessing import Lock
from multiprocessing.managers import BaseManager
from llama_index import SimpleDirectoryReader, GPTVectorStoreIndex, Document, ServiceContext, StorageContext, load_index_from_storage

#builds an example sql database table to test LlamaIndex

from sqlalchemy import(
    create_engine, MetaData, 
    Table, Column, 
    Integer, String, Float, 
    select, insert
    )
from sqlalchemy.orm import relationship, sessionmaker
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
class Data(Base):
    __tablename__ = 'data'
    id      = Column(Integer, primary_key=True, autoincrement=True)
    content = Column(String(100), nullable=False)

def loadDatabase():
    engine = create_engine('sqlite:///database.db') 
    metadata_obj = MetaData()
    metadata_obj.create_all(engine)
    Session = sessionmaker(bind=engine)
    Base.metadata.create_all(engine)

    return Session(), engine

def autoPopulate(session):
    list = ["monkey", "chimp", "spider", "gorilla", "iguana", "LORIS", "ant", "squirrel", "sharkey", "penguin"]
    for i in range(10):
        dummy_row = Data(content=list[i])
        session.add(dummy_row)  
    session.commit() 
    
def add(session, data):
    dummy_row = Data(content=data)
    session.add(dummy_row)  
    session.commit()

def remove(session, data):
    rows_to_delete = session.query(Data).filter(Data.content == data).all()

    if rows_to_delete is not None:
        for row in rows_to_delete:
            session.delete(row)
    else:
        print(f"A row containing <<{data}>> could not be found")
    
    session.commit()

def deleteAll(session):
    session.query(Data).delete()
    session.commit()
